@@ -67,10 +67,16 @@ check.boolean <- function(b) # b==0 or b==1 is also ok
         stop0("the ", short.deparse(substitute(b), "given"),
               " argument is not FALSE, TRUE, 0, or 1")
     if(!(is.logical(b) || is.numeric(b)) || is.na(b) || !(b == 0 || b == 1))
-        stop0(short.deparse(substitute(b), "the argument"), "=", b,
+        stop0(short.deparse(substitute(b), "the argument"), "=", as.char(b),
             " but it should be FALSE, TRUE, 0, or 1")
     b != 0 # convert to logical
 }
+is.boolean <- function(b) # b==NA or b==0 or b==1
+{
+    length(b) == 1 &&
+    (is.logical(b) || is.numeric(b)) &&
+    (is.na(b) || b == 0 || b == 1)
+}
 check.classname <- function(object, substituted.object, allowed.classnames)
 {
     err.msg <- quotify(allowed.classnames)
@@ -106,7 +112,7 @@ check.integer.scalar <- function(object, min=NA, max=NA, null.ok=FALSE,
         check.numeric.scalar(object, min, max, null.ok, na.ok, logical.ok,
                              char.ok.msg=char.ok, object.name=object.name)
         if(!is.null(object) && !is.na(object) && object != floor(object))
-                stop.msg()
+            stop.msg()
     }
     object
 }
@@ -216,7 +222,7 @@ check.vec <- function(object, object.name, expected.len=NA, logical.ok=TRUE, na.
 {
     if(!(NROW(object) == 1 || NCOL(object) == 1))
         stop0(object.name, " is not a vector\n       ",
-              object.name, " has dimensions ", NROW(object), " by ", NCOL(object))
+              "It has dimensions ", NROW(object), " by ", NCOL(object))
     if(!((logical.ok && is.logical(object)) || is.numeric(object)))
         stop0(object.name, " is not numeric")
     if(!is.na(expected.len) && length(object) != expected.len)
@@ -228,6 +234,15 @@ check.vec <- function(object, object.name, expected.len=NA, logical.ok=TRUE, na.
         check(object, object.name, "NA", is.na)
     check(object, object.name, "non-finite value", function(object) {!is.finite(object)})
 }
+cleantry <- function(err) # clean up a try.err (remove "Error: " etc.)
+{
+    stopifnot(is.try.err(err))
+    attributes(err) <- NULL
+    err <- gsub("^[^:]*: *", "", err) # remove "Error: " (actually everything up to the first colon)
+    err <- gsub("\n", " ", err, fixed=TRUE)  # remove newlines
+    err <- gsub("  +", " ", err)             # multiple spaces to single spaces
+    gsub(" $", "", err)                      # remove trailing space
+}
 # returns the column name, if that is not possible then something like x[,1]
 colname <- function(object, i, object.name=trunc.deparse(substitute(object)))
 {
@@ -390,7 +405,7 @@ get.model.env <- function(object, object.name="object", trace=0)
         if(is.null(call))
             printf("object class is \"%s\" with no call\n", class(object)[1])
         else
-            printf.wrap("object call is %s\n", strip.deparse(call))
+            printf.wrap("object call is %s\n", strip.deparse(call), maxlen=80)
         printf("--get.model.env for %s object\n", class(object)[1])
     }
     # following will fail for non-formula models because they have no terms field
@@ -447,8 +462,8 @@ get.weighted.rsq <- function(y, yhat, w=NULL) # NAs will be dropped before calc
         yhat <- yhat[!is.na]
         if(length(y) == 0)
             stop0("length(y) == 0 after deleting NAs in y or yhat")
-        rss <- sum((y - yhat)^2)
-        tss <- sum((y - mean(y))^2)
+        rss <- sos(y - yhat)
+        tss <- sos(y - mean(y))
     } else {
         stopifnot(length(w) == length(yhat))
         is.na <- is.na(y) | is.na(yhat) | is.na(w)
@@ -457,8 +472,8 @@ get.weighted.rsq <- function(y, yhat, w=NULL) # NAs will be dropped before calc
         w    <- w[!is.na]
         if(length(y) == 0)
             stop0("length(y) == 0 after deleting NAs in y or yhat or w")
-        rss <- sum(w * (y - yhat)^2)
-        tss <- sum(w * (y - weighted.mean(y, w))^2)
+        rss <- sos(y - yhat, w)
+        tss <- sos(y - weighted.mean(y, w), w)
     }
     get.rsq(rss, tss)
 }
@@ -572,7 +587,7 @@ lighten <- function(col, lighten.amount, alpha=1)
     else
         rgb(rgb[1,], rgb[2,], rgb[3,], alpha)
 }
-# returns the expanded arg
+# returns the expanded arg (error msg if arg is not an allowed choice in calling func)
 match.arg1 <- function(arg, argname=deparse(substitute(arg)))
 {
     formal.args <- formals(sys.function(sys.parent()))
@@ -580,10 +595,11 @@ match.arg1 <- function(arg, argname=deparse(substitute(arg)))
     formal.argnames[imatch.choices(arg[1], formal.argnames, argname)]
 }
 # returns a string, choices is a vector of strings
+# error msg if arg is not an allowed choice
 match.choices <- function(arg,
             choices,
             argname=deparse(substitute(arg)),
-            err.msg="",              # error message, "" for automatic
+            err.msg="",              # error message ("" for automatic)
             err.msg.ext="")          # extension to error message
 {
     choices[imatch.choices(arg, choices, argname,
@@ -686,7 +702,7 @@ pastef <- function(s, fmt, ...) # paste the printf style args to s
 {
     paste0(s, sprint(fmt, ...))
 }
-print.first.few.elements.of.vector <- function(x, trace, name=NULL)
+print_first_few_elements_of_vector <- function(x, trace, name=NULL)
 {
     try(cat(" min", min(x), "max", max(x)), silent=TRUE)
     spaces <- "               "
@@ -865,6 +881,15 @@ sort.unique <- function(x)
 {
     sort(unique(x), na.last=NA) # na.last=NA drops NAs
 }
+sos <- function(x, weights=NULL) # sum of squares
+{
+    if(is.null(weights))
+        sum(as.vector(x^2))
+    else {
+        stopifnot(length(weights) == length(x))
+        sum(weights * as.vector(x^2))
+    }
+}
 stop0 <- function(...)
 {
     stop(..., call.=FALSE)
@@ -886,6 +911,8 @@ stopifnot.string <- function(s, name=short.deparse(substitute(s)),
     if(!is.character(s))
         stop0("'", name, "' is not a character variable (class(",
               name, ") is \"", class(s), "\")")
+    if(length(s) == 0)
+        stop0("'", name, "' is empty (it has no elements)")
     if(length(s) != 1)
         stop0("'", name, "' has more than one element\n       ",
               name, " = c(", paste.trunc("\"", s, "\"", sep=""), ")")
@@ -946,12 +973,18 @@ text.on.white <- function(x, y, label,
     else
         stop0("srt=", srt, " is not allowed (only 0, 90, and -90 are supported)")
 }
-to.logical <- function(object, len)
+to.logical <- function(object, len) # object can be a boolean or numeric vector
 {
     xlogical <- repl(FALSE, len)
     xlogical[object] <- TRUE
     xlogical
 }
+trace0 <- function(trace, fmt, ...)
+{
+    stopifnot(!(is.numeric(trace) && is.logical(trace)))
+    if(trace >= 0)
+        cat(sprint(fmt, ...), sep="")
+}
 trace1 <- function(trace, fmt, ...)
 {
     stopifnot(!(is.numeric(trace) && is.logical(trace)))
@@ -1027,3 +1060,43 @@ warning0 <- function(...)
 {
     warning(..., call.=FALSE)
 }
+# Binomial pairs response: fraction true for each row.
+#
+# This function is used by both earth and plotmo.
+# If you change it here, change it there too.
+#
+# The first column of y is considered to be "true", the second "false".
+#
+# Example y:
+#   survived died
+#      1        1
+#      0        0   # both values zero
+#      3        4
+#
+# becomes:
+#   survived
+#         .5        # 1 / (1 + 1)
+#          0        # special case (both survived and died equal to 0)
+#        .43        # 3 / (3 + 4)
+
+bpairs.yfrac <- function(y, trace)
+{
+    stopifnot(NCOL(y) == 2)
+    both.zero <- (y[,1] == 0) & (y[,2] == 0)
+    y[both.zero, 1] <- 1 # so zero rows will be translated to 0 in next line
+    yfrac <- y[, 1, drop=FALSE] / (y[,1] + y[,2]) # fraction true
+    trace.bpairs.yfrac(yfrac, trace)
+    yfrac
+}
+trace.bpairs.yfrac <- function(yfrac, trace)
+{
+    # based on code in print.earth.fit.args
+    if(trace >= 4)
+        cat("\n")
+    if(trace >= 1 && trace < 7) { # don't print matrices when doing very detailed earth.c tracing
+        tracex <- if(trace >= 5) 4 else 2 # adjust trace for print_summary
+        details <- if(trace >= 4) 2 else if(trace >= 1) -1 else 0
+        print_summary(yfrac, "yfrac", tracex, details=details)
+        if(details > 1) printf("\n")
+    }
+}
